<!--
 Copyright (c) 2024 laffra - All Rights Reserved. 
 
 This file is part of the PySheets projects and contains the HTML for rendering the applicaiton UI.
-->

<!DOCTYPE html>
<html lang="en">

<head>
    <title>PySheets</title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">

    <link rel="stylesheet" href="https://pyscript.net/releases/2024.7.1/core.css">
    <script type="module" src="https://pyscript.net/releases/2024.7.1/core.js"></script>

    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.7.1/jquery.min.js" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.13.2/jquery-ui.min.js" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.13.2/themes/base/jquery-ui.min.css" crossorigin="anonymous" referrerpolicy="no-referrer" />

    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.35.0/codemirror.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/codemirror/6.65.7/mode/python/python.min.js"></script>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/codemirror/6.65.7/codemirror.min.css">

    <script defer src="leader-line.min.js"></script>
    <script defer src="html2canvas.min.js"></script>
    <script src="pysheets.js" type="module"></script>
    <script>
        window.version_pyscript = '2024.7.1';
        window.version_interpreter = '{{version_interpreter}}';
        window.runtime = '{{runtime}}';
        window.editor_width = parseFloat('{{editor_width}}');
        window.timestamp = parseFloat('{{timestamp|safe}}');
        window.path = '{{path}}';
    </script>
    <link defer rel="stylesheet" href="pysheets.css">

    <{{runtime}}-config>
        {{interpreter|safe}}
        {{auto|safe}}
        {{packages|safe}}
        [files]
        {{files|safe}}
    </{{runtime}}-config>
    <script type="{{runtime}}">
        import pysheets
        pysheets.main()
    </script>
    <style>
        {{css}}
    </style>
</head>
<body>
    <div class="header">
        <div class="logo-container">
            <img class="logo" src="favicon.ico">
            <div id="menu" class="menu"></div>
        </div>
        <div id="cell-attributes-container" class="cell-attributes-container"></div>
        <div id="info" class="info">
            <input id="title" value={{loading}}>
        </div>
        <div class="column buttons-container"></div>
        <div class="column user-image-container">
        </div>
    </div>
    <div id="main" class="main" tabindex="0">
    {{sheet_html|safe}}
    </div>
</body>

</html>